;/*Programm zum Umsetzen eines Proportionalkanals auf einen 8-fach Schaltkanal auf 12c508 mit internem Oszillator
;Impulseingang geht auf GPIO3
;Binaere Ausgaenge liegen auf GPIO2-0
;
;Schaltstellung 0: von 0.50ms bis 1.07ms Pulsdauer, Timerwert 000 bis 071, Ausgang 000
;Schaltstellung 1: von 1.07ms bis 1.21ms Pulsdauer, Timerwert 072 bis 089, Ausgang 001
;Schaltstellung 2: von 1.21ms bis 1.35ms Pulsdauer, Timerwert 090 bis 106, Ausgang 010
;Schaltstellung 3: von 1.35ms bis 1.49ms Pulsdauer, Timerwert 107 bis 124, Ausgang 011
;Schaltstellung 4: von 1.49ms bis 1.63ms Pulsdauer, Timerwert 125 bis 141, Ausgang 100
;Schaltstellung 5: von 1.63ms bis 1.78ms Pulsdauer, Timerwert 142 bis 160, Ausgang 101
;Schaltstellung 6: von 1.78ms bis 1.92ms Pulsdauer, Timerwert 161 bis 178, Ausgang 110
;Schaltstellung 7: von 1.92ms bis 2.53ms Pulsdauer, Timerwert 179 bis 254, Ausgang 111
;Undefiniert     : von 2,53ms aufwrts Pulsdauer,   Timerwert 255+       , Ausgang 000
;
;Interner timer wird geclockt mit 4MHz/4 = 1Mhz, prescaler ist auf 1:8
;-> resolution ist 0,008ms, maximaler Zaehlerstand ist 2.04ms
;
;Config-register sollte bei Programmierung auf xxxxxxx01010 (0x00A) stehen
;(mclr=0: GPIO3 anstatt MCLR-pin, cp=1: code protection off, wdt=0: kein watchdog timer, fosc=10: interner oszillator)*/


	processor  12C508A
	radix  DEC
	__CONFIG 0x00A

;Register
TMR      	EQU   	0x01
STATUS    	EQU   	0x03
OSCCAL      EQU   	0x05
GPIO        EQU   	0x06

;Registerbits
_C       	EQU   	0		;carry in STATUS
_Z       	EQU   	2		;zero in STATUS
_W			EQU		0		;destination W
_R			EQU		1		;destination self-register

;Defines
#define 	PulsIn	GPIO,3

;Variablen
mem			EQU		0x07
n           EQU   	mem
m           EQU   	mem+1
zw    		EQU   	mem+2

;Konstanten
zw1			EQU		.72		;Messwert, ab dem Ausgabe 1 ist
zw2			EQU		.90		;Messwert, ab dem Ausgabe 2 ist
zw3			EQU		.107	;Messwert, ab dem Ausgabe 3 ist
zw4			EQU		.125	;Messwert, ab dem Ausgabe 4 ist
zw5			EQU		.142	;Messwert, ab dem Ausgabe 5 ist
zw6			EQU		.161	;Messwert, ab dem Ausgabe 6 ist
zw7			EQU		.179	;Messwert, ab dem Ausgabe 7 ist

	ORG 0
		MOVWF	OSCCAL		;OSCCAL mit vorprogrammiertem Wert belegen
		GOTO 	main

;**** Wait 0,5ms *****************
;Zusammen mit dem aufrufenden Call werden hier 500 Zyklen = 0,5ms gewartet
w05ms	MOVLW	.123
		MOVWF	n
		NOP
		DECFSZ	n,_R
		GOTO 	$-2
		NOP
		NOP
		NOP
		RETLW	.0
;*********************************


main	CLRF  	TMR
		CLRF  	GPIO
		MOVLW 	0xC2 
		OPTION				;wake-up disabled, weak pullup disabled, timer input intern, prescaler assigned to timer, prescale by 1:8
		MOVLW	0x38
		TRIS	GPIO		;GP0-2 sind Ausgaenge, GP3-5 Eingange

start	BTFSC 	PulsIn		;Warte auf Ende eines evtl anstehenden Pulses
		GOTO  	$-1
		BTFSS 	PulsIn		;Warte auf Beginn eines Pulses
		GOTO  	$-1
		;Ein Pulsstart wurde detektiert
		CALL	w05ms		;0,5ms warten				
		CLRF  	TMR			;Zeitmessung starten		
		BTFSS 	PulsIn		;Warte auf Ende des Pulses oder Zhlerberlauf 
		GOTO	exit		;Pulsende -> Auswertung
		INCF	TMR,_W		;Puls noch aktiv -> W=TMR+1
		BTFSS	STATUS,_Z	;Zero? Bzw. ist TMR=255?
		GOTO  	$-4			;nein: weiterwarten
		CLRF	GPIO		;ja: Ausgabe 0
		GOTO	start
		;Der Puls ist zu Ende	
exit	MOVF  	TMR,_W		;Pulsdauer speichern
		MOVWF 	zw
		;Pulsdauer auswerten und Ausgaenge entsprechend setzen
test0	MOVLW 	zw1	
		SUBWF 	zw,_W		;W=zw-zw1
		BTFSC 	STATUS,_C	;Borrow aufgetreten (zw<zw1)?
		GOTO  	test1		;nein: weitertesten
		CLRF  	GPIO		;ja: Ausgabe 0
		GOTO  	start
test1	MOVLW 	zw2	
		SUBWF 	zw,_W		;W=zw-zw2
		BTFSC 	STATUS,_C	;Borrow aufgetreten (zw<zw2)?
		GOTO  	test2		;nein: weitertesten
		MOVLW	0x01
		MOVWF	GPIO		;ja: Ausgabe 1
		GOTO  	start
test2	MOVLW 	zw3	
		SUBWF 	zw,_W		;W=zw-zw3
		BTFSC 	STATUS,_C	;Borrow aufgetreten (zw<zw3)?
		GOTO  	test3		;nein: weitertesten
		MOVLW	0x02
		MOVWF	GPIO		;ja: Ausgabe 2
		GOTO  	start
test3	MOVLW 	zw4	
		SUBWF 	zw,_W		;W=zw-zw4
		BTFSC 	STATUS,_C	;Borrow aufgetreten (zw<zw4)?
		GOTO  	test4		;nein: weitertesten
		MOVLW	0x03
		MOVWF	GPIO		;ja: Ausgabe 3
		GOTO  	start
test4	MOVLW 	zw5	
		SUBWF 	zw,_W		;W=zw-zw5
		BTFSC 	STATUS,_C	;Borrow aufgetreten (zw<zw5)?
		GOTO  	test5		;nein: weitertesten
		MOVLW	0x04
		MOVWF	GPIO		;ja: Ausgabe 4
		GOTO  	start
test5	MOVLW 	zw6	
		SUBWF 	zw,_W		;W=zw-zw6
		BTFSC 	STATUS,_C	;Borrow aufgetreten (zw<zw6)?
		GOTO  	test6		;nein: weitertesten
		MOVLW	0x05
		MOVWF	GPIO		;ja: Ausgabe 5
		GOTO  	start
test6	MOVLW 	zw7	
		SUBWF 	zw,_W		;W=zw-zw7
		BTFSC 	STATUS,_C	;Borrow aufgetreten (zw<zw7)?
		GOTO  	out7		;nein: Ausgabe 7
		MOVLW	0x06
		MOVWF	GPIO		;ja: Ausgabe 6
		GOTO  	start
out7	MOVLW	0x07
		MOVWF	GPIO		;Ausgabe 7
		GOTO	start

		END
